<?php

if (!function_exists('curl_init')) {
  throw new Exception('UOC needs the CURL PHP extension.');
}
if (!function_exists('json_decode')) {
  throw new Exception('UOC needs the JSON PHP extension.');
}

/**
 * Thrown when an API call returns an exception.
 *
 * @author 
 */
class UOCApiException extends Exception
{
  /**
   * The result from the API server that represents the exception information.
   */
  protected $result;

  /**
   * Make a new API Exception with the given result.
   *
   * @param Array $result the result from the API server
   */
  public function __construct($result) {
    $this->result = $result;

    $code = isset($result['error_code']) ? $result['error_code'] : 0;

    if (isset($result['error_description'])) {
      // OAuth 2.0 Draft 10 style
      $msg = $result['error_description'];
    } else if (isset($result['error']) && is_array($result['error'])) {
      // OAuth 2.0 Draft 00 style
      $msg = $result['error']['message'];
    } else if (isset($result['error_msg'])) {
      // Rest server style
      $msg = $result['error_msg'];
    } else {
      $msg = 'Unknown Error. Check getResult()';
    }

    parent::__construct($msg, $code);
  }

  /**
   * Return the associated result object returned by the API server.
   *
   * @returns Array the result from the API server
   */
  public function getResult() {
    return $this->result;
  }

  /**
   * Returns the associated type for the error. This will default to
   * 'Exception' when a type is not available.
   *
   * @return String
   */
  public function getType() {
    if (isset($this->result['error'])) {
      $error = $this->result['error'];
      if (is_string($error)) {
        // OAuth 2.0 Draft 10 style
        return $error;
      } else if (is_array($error)) {
        // OAuth 2.0 Draft 00 style
        if (isset($error['type'])) {
          return $error['type'];
        }
      }
    }
    return 'Exception';
  }

  /**
   * To make debugging easier.
   *
   * @returns String the string representation of the error
   */
  public function __toString() {
    $str = $this->getType() . ': ';
    if ($this->code != 0) {
      $str .= $this->code . ': ';
    }
    return $str . $this->message;
  }
}

/**
 * Provides access to the UOC Platform
 *
 * 
 */
class UOC
{
  /**
   * The Client ID.
   */
  protected $clientId;
  
  /**
   * The secret Key
   */
  protected $secretKey;
  
  
  /**
   * Host 
   */
  protected $host;
 

  /**
   * Initialize (Constructor).
   *
   * The configuration:
   * - clientId : clientId required for UOCAPI
   * - secretKey: secretKey required for UOCAPI
   * - host: url to UOCAPI Host
   *
   * @param Array $config the application configuration
   */
  public function __construct($config) {
    $this->clientId = $config['clientId']; 
	$this->secretKey = $config['secret'];
	$this->host = $config['host'];
	
	error_log('oscar: Secret ' .$this->secretKey );
  }

 

   /**
   * Per crides a la API amb curl
   *
   * @return String amb la resposta del servidor
   */
  protected function call_API($url) {
	  $curl = curl_init($url);
	  curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
	  $curl_response = curl_exec($curl);
	  
	  if ($curl_response === false) {
           $info = curl_getinfo($curl);
           curl_close($curl);
           error_log('oscar: error occured during curl exec. Additioanl info: ' . var_export($info));
	 } 
	  
	  curl_close($curl);
	  $decoded = json_decode($curl_response);
	  
	  if (isset($decoded->response->status) && $decoded->response->status == 'ERROR') {
          error_log('oscar error occured: ' . $decoded->response->errormessage);
      }
	  
	  return $curl_response;
	  
  }
 
  
   /**
   * Construeix la url per demanar codi autoritzacio 
   * @param $return_url url de retorn
   * @return String url
   */
  public function getLoginUrl($return_url) {

	  $authUrl = $this->host . '/oauth/authorize'
	           .'?response_type=code' 
	           .'&scope=READ'
			   .'&client_id=' . $this->clientId
			   .'&state=authorize'
			   .'&redirect_uri=' . $return_url;
			 
	  return $authUrl;
  }
  
  
   /**
   * Retorna el valor del parametre code de la url o false si no el troba
   */
   protected function getCode() {
	  
    if (isset($_REQUEST['code'])) {
        return $_REQUEST['code'];
    } else {
        return false;
    }
    return false;
  }
  
   /**
   * Obte el token. Previament necessitem haver obtingut el code com a resposta a /oauth/authorize
   *
   * @param $return_url url de retorn
   * @return String token, false en cas que no s'hagi pogut obtenir
   */
  protected function getToken($return_url) {
    
	$code = $this->getCode();  //obtenim el codi
	
	if (!$code) {
		error_log('oscar: No hi ha code...no podem obtenir token');
		return false;
	}
	
	if (!$code) {
	  return false;
	} else {
     
	$tokenUrl = $this->host . '/oauth/token'
	           .'?grant_type=authorization_code'
	           .'&client_id=' . $this->clientId
			   .'&client_secret=' . $this->secretKey
			   .'&code='  . $code
			   .'&scope=READ'
			   .'&redirect_uri=' . $return_url;
		   
	  //Javier el forward diria que no es pot fer servir...ja que es una redireccio i l'executa en el navegador...jo vull una crida dins de elgg com quan faig crides a Api Rest amb ajax...
	  //forward($tokenUrl,'uocKpax_api'); 
	  
	  // he implementat una funcio senzilla per a que faci la crida amb curl
	  $token=$this->call_API($tokenUrl);  
	
      error_log('oscar: getToken -> respuesta API /oauth/token = ' . $token);
	  
	  $decoded = json_decode($token); //ho passem a json
	  
	  if (isset($decoded->access_token)) { //comprovem si tenim el token
		   $acces_token = $decoded->access_token;
	       error_log('oscar: getToken -> obtenemos el token = ' . $acces_token);
		   return  $acces_token; 
	  }
	}	 
	return false;
  }
  
  protected static $kSupportedKeys =
    array('state', 'code', 'access_token', 'username', 'name', 'email' );

  /**
   * Provides the implementations of the inherited abstract
   * methods.  The implementation uses PHP sessions to maintain
   * a store for authorization codes, user ids, CSRF states, and
   * access tokens.
   */
  protected function setPersistentData($key, $value) {
    if (!in_array($key, self::$kSupportedKeys)) {
      self::errorLog('Unsupported key passed to setPersistentData.');
      return;
    }

    $session_var_name = $this->constructSessionVariableName($key);
    $_SESSION[$session_var_name] = $value;
  }

  protected function getPersistentData($key, $default = false) {
    if (!in_array($key, self::$kSupportedKeys)) {
      self::errorLog('Unsupported key passed to getPersistentData.');
      return $default;
    }

    $session_var_name = $this->constructSessionVariableName($key);
    return isset($_SESSION[$session_var_name]) ?
      $_SESSION[$session_var_name] : $default;
  }

  protected function clearPersistentData($key) {
    if (!in_array($key, self::$kSupportedKeys)) {
      self::errorLog('Unsupported key passed to clearPersistentData.');
      return;
    }

    $session_var_name = $this->constructSessionVariableName($key);
    unset($_SESSION[$session_var_name]);
  }

  protected function clearAllPersistentData() {
    foreach (self::$kSupportedKeys as $key) {
      $this->clearPersistentData($key);
    }
  }

  protected function constructSessionVariableName($key) {
    return implode('_', array('uoc',
                               $this->clientId,
                              $key));
  }
  
  
  
  
  
  
  /**
    * Obtenim les dades de l'usuari UOC  
	* @return JSON amb les dades de usuari UOC
    */
  public function getUser($return_url='') {
	  $acces_token = $this->getToken($return_url);
	  
	  $userUrl =  $this->host . '/api/v1/user'
	           .'?access_token=' .$acces_token;
	  
	  error_log('oscar: getUser -> llamada userUrl = ' . $userUrl );
	  
	  $userUOC = $this->call_API($userUrl);
	  
	  $decoded = json_decode($userUOC); 
	  
	  error_log('oscar: getUser -> respuesta userUrl = ' . $userUOC );
	  
	  return $decoded;
	  
  }
  
}
